@@ -10,7 +10,7 @@ from textwrap import dedent
 from cStringIO import StringIO
 
 from ceph_deploy import conf, exc, hosts
-from ceph_deploy.util import constants
+from ceph_deploy.util import constants, system
 from ceph_deploy.cliutil import priority
 from ceph_deploy.lib import remoto
 
@@ -71,8 +71,9 @@ def osd_tree(conn, cluster):
     Note how the booleans are actually strings, so we need to take that into
     account and fix it before returning the dictionary. Issue #8108
     """
+    ceph_executable = system.executable_path(conn, 'ceph')
     command = [
-        'ceph',
+        ceph_executable,
         '--cluster={cluster}'.format(cluster=cluster),
         'osd',
         'tree',
@@ -200,7 +201,9 @@ def prepare_disk(
     Run on osd node, prepares a data disk for use.
     """
     args = [
-        'ceph-disk-prepare',
+        'ceph-disk',
+        '-v',
+        'prepare',
         ]
     if zap:
         args.append('--zap-disk')
@@ -241,6 +244,31 @@ def prepare_disk(
         )
 
 
+def exceeds_max_osds(args, reasonable=20):
+    """
+    A very simple function to check against multiple OSDs getting created and
+    warn about the possibility of more than the recommended which would cause
+    issues with max allowed PIDs in a system.
+
+    The check is done against the ``args.disk`` object that should look like::
+
+        [
+            ('cephnode-01', '/dev/sdb', '/dev/sda5'),
+            ('cephnode-01', '/dev/sdc', '/dev/sda6'),
+            ...
+        ]
+    """
+    hosts = [item[0] for item in args.disk]
+    per_host_count = dict(
+        (
+            (h, hosts.count(h)) for h in set(hosts)
+            if hosts.count(h) > reasonable
+        )
+    )
+
+    return per_host_count
+
+
 def prepare(args, cfg, activate_prepared_disk):
     LOG.debug(
         'Preparing cluster %s disks %s',
@@ -248,6 +276,14 @@ def prepare(args, cfg, activate_prepared_disk):
         ' '.join(':'.join(x or '' for x in t) for t in args.disk),
         )
 
+    hosts_in_danger = exceeds_max_osds(args)
+
+    if hosts_in_danger:
+        LOG.warning('if ``kernel.pid_max`` is not increased to a high enough value')
+        LOG.warning('the following hosts will encounter issues:')
+        for host, count in hosts_in_danger.items():
+            LOG.warning('Host: %8s, OSDs: %s' % (host, count))
+
     key = get_bootstrap_osd_key(cluster=args.cluster)
 
     bootstrapped = set()
@@ -333,7 +369,9 @@ def activate(args, cfg):
         remoto.process.run(
             distro.conn,
             [
-                'ceph-disk-activate',
+                'ceph-disk',
+                '-v',
+                'activate',
                 '--mark-init',
                 distro.init,
                 '--mount',
@@ -343,11 +381,14 @@ def activate(args, cfg):
         # give the OSD a few seconds to start
         time.sleep(5)
         catch_osd_errors(distro.conn, distro.conn.logger, args)
+
+        if distro.is_el:
+            system.enable_service(distro.conn)
+
         distro.conn.exit()
 
 
 def disk_zap(args):
-    cfg = conf.ceph.load(args)
 
     for hostname, disk, journal in args.disk:
         if not disk or not hostname:
@@ -361,29 +402,44 @@ def disk_zap(args):
             distro.codename
         )
 
-        # NOTE: this mirrors ceph-disk-prepare --zap-disk DEV
-        # zero the device
         distro.conn.remote_module.zeroing(disk)
 
+        ceph_disk_executable = system.executable_path(distro.conn, 'ceph-disk')
         remoto.process.run(
             distro.conn,
             [
-                'sgdisk',
-                '--zap-all',
-                '--',
-                disk,
-            ],
-        )
-        remoto.process.run(
-            distro.conn,
-            [
-                'sgdisk',
-                '--clear',
-                '--mbrtogpt',
-                '--',
+                ceph_disk_executable,
+                'zap',
                 disk,
             ],
         )
+
+        # once all is done, call partprobe (or partx)
+        # On RHEL and CentOS distros, calling partprobe forces a reboot of the
+        # server. Since we are not resizing partitons we rely on calling
+        # partx
+        if distro.normalized_name.startswith(('centos', 'red')):
+            LOG.info('calling partx on zapped device %s', disk)
+            LOG.info('re-reading known partitions will display errors')
+            remoto.process.run(
+                distro.conn,
+                [
+                    'partx',
+                    '-a',
+                    disk,
+                ],
+            )
+
+        else:
+            LOG.debug('Calling partprobe on zapped device %s', disk)
+            remoto.process.run(
+                distro.conn,
+                [
+                    'partprobe',
+                    disk,
+                ],
+            )
+
         distro.conn.exit()
 
 
@@ -398,10 +454,11 @@ def disk_list(args, cfg):
         )
 
         LOG.debug('Listing disks on {hostname}...'.format(hostname=hostname))
+        ceph_disk_executable = system.executable_path(distro.conn, 'ceph-disk')
         remoto.process.run(
             distro.conn,
             [
-                'ceph-disk',
+                ceph_disk_executable,
                 'list',
             ],
         )
@@ -433,10 +490,11 @@ def osd_list(args, cfg):
         remote_module = distro.conn.remote_module
         osds = distro.conn.remote_module.listdir(constants.osd_path)
 
+        ceph_disk_executable = system.executable_path(distro.conn, 'ceph-disk')
         output, err, exit_code = remoto.process.check(
             distro.conn,
             [
-                'ceph-disk',
+                ceph_disk_executable,
                 'list',
             ]
         )
@@ -528,7 +586,6 @@ def print_osd(logger, hostname, osd_path, json_blob, metadata, journal=None):
     if journal:
         logger.info('Journal: %s' % journal)
     for k, v in metadata.items():
-        #logger.info("%s: %-8s" % (k.capitalize(), v))
         logger.info("%-13s  %s" % (k.capitalize(), v))
 
     logger.info('-'*40)
